<?php
/********************************************************************
  
   PhPeace - Portal Management System

   Copyright notice
   (C) 2010 Francesco Iannuzzelli <francesco@phpeace.org>
   All rights reserved

   This script is part of PhPeace.
   PhPeace is free software; you can redistribute it and/or modify 
   it under the terms of the GNU General Public License as 
   published by the Free Software Foundation; either version 2 of 
   the License, or (at your option) any later version.

   PhPeace is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   The GNU General Public License (GPL) is available at
   http://www.gnu.org/copyleft/gpl.html.
   A copy can be found in the file COPYING distributed with 
   these scripts.

   This copyright notice MUST APPEAR in all copies of the script!

********************************************************************/

include_once(SERVER_ROOT."/../classes/db.php");
include_once(SERVER_ROOT."/../classes/file.php");
include_once(SERVER_ROOT."/../classes/cache.php");
include_once(SERVER_ROOT."/../classes/texthelper.php");

/**
 * Extract MPs data from Camera and Senato websites.
 * Manage notification for signatures from mail campaigns. 
 * 
 * @package PhPeace
 * @author Francesco Iannuzzelli <francesco@phpeace.org>
 *
 */
class ParlaMonitor
{
	/**
	 * @var FileManager */
	private $fm;

	/**
	 * @var Cache */
	private $cache;

	/**
	 * @var TextHelper */
	private $th;

	/**
	 * How many notifications (per campaign) to process in each scheduler cycle
	 * @var integer
	 */
	const NotificationsLimit = 5;
	
	/**
	 * URL of MPs page (Camera)
	 * @var string
	 */
	const CameraMPList = "http://www.camera.it/28";
	
	/**
	 * Regex pattern to match MP ID (Camera)
	 * @var string
	 */
	const CameraPatternMPLink = '/29\?shadow_deputato=(\d{1,7})/';
	
	/**
	 * URL of MP page (Camera)
	 * @var string
	 */
	const CameraPageMP = "http://www.camera.it/29?shadow_deputato=";

	/**
	 * URL of MPs page (Camera)
	 * @var string
	 */
	const SenatoPageMPList = "http://www.senato.it/leg/16/BGT/Schede/Attsen/Sen";

	/**
	 * Surname initial letters of MPs (Senato), separated by |
	 * @var string (to explode into array)
	 */
	const SenatoLetters = "a|b|c|d|e|f|g|i|l|m|n|o|p|q|r|s|t|v|z";
	
	/**
	 * Regex pattern to match MP ID (Senato)
	 * @var string
	 */
	const SenatoPatternMPLink = '/link.asp\?tipodoc=sattsen&amp;leg=16&amp;id=(\d{1,7})/';
	
	/**
	 * URL of MP page (Senato)
	 * @var string
	 */
	const SenatoPageMP = "http://www.senato.it/loc/link.asp?tipodoc=sattsen&leg=16&id=";
	
	/**
	 * Initialize local properties 
	 */
	function __construct()
	{
		$this->fm = new FileManager();
		$this->cache = new Cache();
		$this->th = new TextHelper();
		$this->cache->ttl = 10080; // 1 week
	}

	/**
	 * Delete all MPs data for specific house and group
	 * 
	 * @param integer $id_mp_group
	 * @param string $house
	 */
	private function HouseClear($id_mp_group,$house)
	{
		$db = Db::globaldb();
		$house = $db->SqlQuote($house);
		$db->begin();
		$db->lock("mps");
		$res[] = $db->query("DELETE FROM mps WHERE id_mp_group='$id_mp_group' AND house='$house' ");	
		Db::finish( $res, $db);
	}
	
	/**
	 * Import all MPs from Camera into specific group
	 * 
	 * @param integer $id_mp_group
	 */
	public function CameraImport($id_mp_group)
	{
		$house = "Camera";
		$page = $this->cache->Get("webpage",self::CameraMPList);
		$mps_ids_temp = array();
		preg_match_all(self::CameraPatternMPLink,$page,$mps_ids_temp);
		$mps_ids = array_unique($mps_ids_temp[1]);
		
		$this->HouseClear($id_mp_group,$house);
		
		$db = Db::globaldb();
		$house = $db->SqlQuote($house);
		$db->begin();
		$db->lock("mps");
		
		foreach($mps_ids as $mps_id)
		{
			$page = $this->cache->Get("webpage",self::CameraPageMP . $mps_id);
			$match = array();
			preg_match('/Deputati - (.*)\<\/title\>/',$page,$match);
			$name = $db->SqlQuote($match[1]);
			$match = array();
			preg_match('/794\?mailer_back_end_recipients=(.*)&amp;shadow_deputato/',$page,$match);
			$email = $db->SqlQuote($match[1]);
			$match = array();
			preg_match('/\<div class="circoscrizione" \>\n(.*)\<\/div\>/',$page,$match);
			$constituency = $match[1];
			$id_geo = $this->Region($constituency);
			$constituency = $db->SqlQuote($constituency);
			$match = array();
			preg_match('/<a href="217\?idGruppo=(.*)" title="(.*)" \>/',$page,$match);
			$party = $db->SqlQuote($match[2]);
			$match = array();
			preg_match('/Cessato dal mandato parlamentare il \<\/div\>(.*)\<\/div\>/',$page,$match);
			$ended = $match[1];
		
			if($ended=="")
			{
				$sqlstr = "INSERT INTO mps (`id_mp_group`,`id_geo`,`name`,`email`,`constituency`,`house`,`party`) 
					VALUES ($id_mp_group,$id_geo,'$name','$email','$constituency','$house','$party') ";
				$res[] = $db->query( $sqlstr );	
			}
		}
		Db::finish( $res, $db);
	}

	/**
	 * Retrieve all MPs elected in the specified region
	 * 
	 * @param integer $id_reg
	 * @param integer $id_mp_group
	 * @return array:
	 */
	public function MPsByRegion($id_reg,$id_mp_group)
	{
		$db =& Db::globaldb();
		$rows = array();
		$sqlstr = "SELECT name,email FROM mps WHERE id_mp_group='$id_mp_group' AND id_geo='$id_reg' AND email<>'' ";
		$db->QueryExe($rows, $sqlstr);
		return $rows;
	}
	
	/**
	 * Remove a notification job
	 * 
	 * @param integer $id_mps_queue
	 */
	private function NotificationDelete($id_mps_queue)
	{
		$db =& Db::globaldb();
		$db->begin();
		$db->lock( "mps_queue" );
		$res[] = $db->query( "DELETE FROM mps_queue WHERE id_mps_queue='$id_mps_queue' " );
		Db::finish( $res, $db);
	}
	
	/**
	 * Add a notification job to the queue
	 * 
	 * @param integer $id_topic_campaign
	 * @param integer $id_p
	 * @param integer $id_geo
	 */
	public function NotificationQueueAdd($id_topic_campaign,$id_p,$id_geo)
	{
		include_once(SERVER_ROOT."/../classes/geo.php");
		$geo = new Geo();
		if($geo->geo_location=="1")
		{
			$id_reg = $geo->RegionByProvince($id_geo);
			if($id_reg>0)
			{
				$db =& Db::globaldb();
				$today_time = $db->GetTodayTime();
				$db->begin();
				$db->lock( "mps_queue" );
				$id_mps_queue = $db->nextId( "mps_queue", "id_mps_queue" );
				$sqlstr = "INSERT INTO mps_queue (id_mps_queue,id_p,id_topic_campaign,sign_date)
						VALUES ($id_mps_queue,'$id_p','$id_topic_campaign','$today_time')";
				$res[] = $db->query( $sqlstr );
				Db::finish( $res, $db);				
			}
		}
	}
	
	/**
	 * Execute notification jobs
	 *  
	 * @return array:
	 */
	public function NotifyMPs()
	{
		$db =& Db::globaldb();
		$campaigns = array();
		$sqlstr = "SELECT DISTINCT id_topic_campaign FROM mps_queue ";
		$db->QueryExe($campaigns, $sqlstr);
		$messages = array();
		$mail_counter = 0;
		$notification_counter = 0;
		if(count($campaigns)>0)
		{
			include_once(SERVER_ROOT."/../classes/mail.php");
			include_once(SERVER_ROOT."/../classes/irl.php");
			include_once(SERVER_ROOT."/../classes/translator.php");
			include_once(SERVER_ROOT."/../classes/topic.php");
			include_once(SERVER_ROOT."/../classes/geo.php");
			include_once(SERVER_ROOT."/../classes/campaign.php");
			include_once(SERVER_ROOT."/../classes/people.php");
			include_once(SERVER_ROOT."/../classes/log.php");
			$log = new Log();
			$mail = new Mail();
			$irl = new IRL();
			$geo = new Geo();
			$pe = new People();
			foreach($campaigns as $campaign)
			{
				$c = new Campaign($campaign['id_topic_campaign']);
				$campaign = $c->CampaignGet();
				$t = new Topic($campaign['id_topic']);
				$url = $irl->PublicUrlGlobal("campaign",array('id'=>$c->id,'id_topic'=>($campaign['id_topic'])));
				$subject = $campaign['name'];
				$promoter = $c->CampaignPromoter($campaign,$t);
				$tr = new Translator($t->id_language,4,false,$t->id_style);
				
				$notifications = array();
				$sqlstr = "SELECT id_mps_queue,id_p FROM mps_queue WHERE id_topic_campaign='{$campaign['id_topic_campaign']}' ORDER BY sign_date DESC LIMIT " . self::NotificationsLimit;
				$num_notifications = $db->QueryExe($notifications, $sqlstr);
					
				foreach($notifications as $notification)
				{
					$user = $pe->UserGetDetailsById($notification['id_p']);
					$id_reg = $geo->RegionByProvince($user['id_geo']);
					$mps = $this->MPsByRegion($id_reg,$campaign['id_mp_group']);
					$signer_name = trim($user['name1'] . " " . $user['name2']);
					$signer_email = $user['email'];
					$message = $campaign['notify_text'] . "\n\n$signer_name\n$signer_email\n\n-----------\n" . $tr->TranslateParams("notification_footer",array($campaign['name'],$url,"{$promoter['name']} <{$promoter['email']}>"));
					$extra = array('name'=>$signer_name,'email'=>$signer_email);
					foreach($mps as $mp)
					{
						$mail->SendMail($mp['email'],$mp['name'],$subject,$message,$extra,false,array(),false);
						$mail_counter++;
					}
					$this->NotificationDelete($notification['id_mps_queue']);
					$notification_counter++;
				}
				$log_message = "Sent $mail_counter messages for $notification_counter notification jobs, as part of campaign \"{$campaign['name']}\"";
				$log->Write("info",$log_message);
				$messages[] = $log_message;
			}
		}
		return $messages;
	}
	
	/**
	 * Lookup region ID from Constituency name
	 * 
	 * @param string $constituency
	 * @return integer
	 */
	private function Region($constituency)
	{
		$id_geo = 0;
		$constituency = strtoupper($constituency);
		if(strpos($constituency,"VALLE D'AOSTA")!==false)
			$id_geo = 1;
		if(strpos($constituency,"PIEMONTE")!==false)
			$id_geo = 2;
		if(strpos($constituency,"LOMBARDIA")!==false)
			$id_geo = 3;
		if(strpos($constituency,"VENETO")!==false)
			$id_geo = 4;
		if(strpos($constituency,"TRENTINO")!==false)
			$id_geo = 5;
		if(strpos($constituency,"FRIULI")!==false)
			$id_geo = 6;
		if(strpos($constituency,"LIGURIA")!==false)
			$id_geo = 7;
		if(strpos($constituency,"TOSCANA")!==false)
			$id_geo = 8;
		if(strpos($constituency,"EMILIA")!==false)
			$id_geo = 9;
		if(strpos($constituency,"MARCHE")!==false)
			$id_geo = 10;
		if(strpos($constituency,"UMBRIA")!==false)
			$id_geo = 11;
		if(strpos($constituency,"LAZIO")!==false)
			$id_geo = 12;
		if(strpos($constituency,"ABRUZZO")!==false)
			$id_geo = 13;
		if(strpos($constituency,"MOLISE")!==false)
			$id_geo = 14;
		if(strpos($constituency,"CAMPANIA")!==false)
			$id_geo = 15;
		if(strpos($constituency,"BASILICATA")!==false)
			$id_geo = 16;
		if(strpos($constituency,"PUGLIA")!==false)
			$id_geo = 17;
		if(strpos($constituency,"CALABRIA")!==false)
			$id_geo = 18;
		if(strpos($constituency,"SICILIA")!==false)
			$id_geo = 19;
		if(strpos($constituency,"SARDEGNA")!==false)
			$id_geo = 20;
		if(strpos($constituency,"ESTERO")!==false || strpos($constituency,"AMERICA")!==false || strpos($constituency,"EUROPA")!==false || strpos($constituency,"ASIA")!==false)
			$id_geo = 21;
		return $id_geo;
	}
	
	/**
	 * Import all MPs from Senato into specific group
	 * @param integer $id_mp_group
	 */
	public function SenatoImport($id_mp_group)
	{
		$house = "Senato";
		$mps_ids = array();
		$letters = explode("|",self::SenatoLetters);
		foreach($letters as $letter)
		{
			$page = $this->cache->Get("webpage",self::SenatoPageMPList . $letter . ".html");
			$mps_ids_temp = array();
			$matches = preg_match_all(self::SenatoPatternMPLink,$page,$mps_ids_temp);
			$mps_ids_temp_unique = array_unique($mps_ids_temp[1]);
			$mps_ids = array_merge($mps_ids,$mps_ids_temp_unique);
		}
		
		$this->HouseClear($id_mp_group,$house);
		
		$db = Db::globaldb();
		$house = $db->SqlQuote($house);
		$db->begin();
		$db->lock("mps");

		foreach($mps_ids as $mps_id)
		{
			$page = $this->cache->Get("webpage",self::SenatoPageMP .$mps_id);
		
			$match = array();
			preg_match('/\<h1 class="titolo"\>(.*)\<\/h1\>/',$page,$match);
			$name = $db->SqlQuote($match[1]);
			$match = array();
			preg_match("/come '(.*)' presso posta.senato.it/",$page,$match);
			$email = $match[1]!=""? $db->SqlQuote($match[1] . "@posta.senato.it") : "";
			$match = array();
			preg_match('/(Regione di elezione|Circoscrizione estera di elezione): \<strong\>(.*)\<\/strong\>/',$page,$match);
			$constituency = $match[2];
			$id_geo = $this->Region($constituency);
			$constituency = $db->SqlQuote($constituency);
			$match = array();
			preg_match('/<a href="\/loc\/link\.asp\?tipodoc=sgrp&amp;leg=16&amp;id=(.*)">(.*)\<\/a\>/',$page,$match);
			$party = $db->SqlQuote($match[2]);
			$match = array();
			preg_match('/(Dimissioni accettate|Dimissioni per incompatibilit|Deceduto)(.*)il(.*)/',$page,$match);
			$ended = $match[3];
		
			if($ended=="")
			{
				$sqlstr = "INSERT INTO mps (`id_mp_group`,`id_geo`,`name`,`email`,`constituency`,`house`,`party`) 
					VALUES ($id_mp_group,$id_geo,'$name','$email','$constituency','$house','$party') ";
				$res[] = $db->query( $sqlstr );	
			}
		}
		Db::finish( $res, $db);
	}
}
?>
